@@ -44,7 +44,8 @@
 #include <QPushButton>
 #include <QMimeDatabase>
 #include <QCompleter>
-#include <QDirModel>
+#include <QFileSystemModel>
+#include <QSettings>
 
 #include "budget.h"
 #include "eqonomizevalueedit.h"
@@ -54,6 +55,12 @@
 #include <cmath>
 #include <ctime>
 
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
+#	define DATE_TO_MSECS(d) QDateTime(d.startOfDay()).toMSecsSinceEpoch()
+#else
+#	define DATE_TO_MSECS(d) QDateTime(d).toMSecsSinceEpoch()
+#endif
+
 #define ALL_TYPES_ID		5
 
 extern QString last_document_directory;
@@ -95,6 +102,24 @@ ImportCSVDialog::ImportCSVDialog(bool extra_parameters, Budget *budg, QWidget *p
 	layout1->addWidget(typeDescriptionLabel);
 	layout1->addItem(new QSpacerItem(1, 1, QSizePolicy::Minimum, QSizePolicy::Expanding));
 
+	QHBoxLayout *layoutPreset = new QHBoxLayout();
+	presetLabel = new QLabel(tr("Presets:"));
+	layoutPreset->addWidget(presetLabel);
+	presetCombo = new QComboBox();
+	presetCombo->setMinimumContentsLength(20);
+	presetCombo->setEditable(false);
+	QSettings settings;
+	settings.beginGroup("GeneralOptions");
+	presets = settings.value("CSVPresets").toMap();
+	for(QMap<QString, QVariant>::const_iterator it = presets.constBegin(); it != presets.constEnd(); ++it) {
+		presetCombo->addItem(it.key());
+	}
+	if(presetCombo->count() == 0) presetCombo->setEnabled(false);
+	else presetCombo->setCurrentIndex(-1);
+	layoutPreset->addWidget(presetCombo);
+	layoutPreset->addStretch(1);
+	layout1->addLayout(layoutPreset);
+
 	QIFWizardPage *page2 = new QIFWizardPage();
 	page2->setTitle(tr("File Selection"));
 	setPage(1, page2);
@@ -104,7 +129,9 @@ ImportCSVDialog::ImportCSVDialog(bool extra_parameters, Budget *budg, QWidget *p
 	QHBoxLayout *layout2h = new QHBoxLayout();
 	fileEdit = new QLineEdit(page2);
 	QCompleter *completer = new QCompleter(this);
-	completer->setModel(new QDirModel(completer));
+	QFileSystemModel *fsModel = new QFileSystemModel(completer);
+	fsModel->setRootPath(QString());
+	completer->setModel(fsModel);
 	fileEdit->setCompleter(completer);
 	layout2h->addWidget(fileEdit);
 	fileButton = new QPushButton(LOAD_ICON("document-open"), QString(), page2);
@@ -134,9 +161,9 @@ ImportCSVDialog::ImportCSVDialog(bool extra_parameters, Budget *budg, QWidget *p
 	page3->setTitle(tr("Columns Specification"));
 	setPage(2, page3);
 	QGridLayout *layout3 = new QGridLayout(page3);
-	
+
 	int row = 0;
-	
+
 	layout3->addWidget(new QLabel(tr("Date:"), page3), row, 0);
 	dateGroup = new QButtonGroup(this);
 	columnDateButton = new QRadioButton(tr("Column"), page3);
@@ -150,7 +177,7 @@ ImportCSVDialog::ImportCSVDialog(bool extra_parameters, Budget *budg, QWidget *p
 	valueDateButton = new QRadioButton(tr("Value"), page3);
 	dateGroup->addButton(valueDateButton);
 	layout3->addWidget(valueDateButton, row, 3);
-	valueDateEdit = new QDateEdit(QDate::currentDate(), page3);
+	valueDateEdit = new EqonomizeDateEdit(QDate::currentDate(), page3);
 	valueDateEdit->setCalendarPopup(true);
 	valueDateEdit->setEnabled(false);
 	layout3->addWidget(valueDateEdit, row, 4);
@@ -231,6 +258,10 @@ ImportCSVDialog::ImportCSVDialog(bool extra_parameters, Budget *budg, QWidget *p
 	valueAC1Edit->setEditable(false);
 	layout3->addWidget(valueAC1Edit, row, 4);
 	row++;
+	valueAC1IncomeEdit = new QComboBox(page3);
+	valueAC1IncomeEdit->setEditable(false);
+	layout3->addWidget(valueAC1IncomeEdit, row, 4);
+	row++;
 
 	AC2Label = new QLabel(tr("From account:"), page3);
 	layout3->addWidget(AC2Label, row, 0);
@@ -251,9 +282,9 @@ ImportCSVDialog::ImportCSVDialog(bool extra_parameters, Budget *budg, QWidget *p
 	valueAC2Edit->setEditable(false);
 	layout3->addWidget(valueAC2Edit, row, 4);
 	row++;
-	
+
 	if(b_extra) {
-	
+
 		quantityLabel = new QLabel(tr("Quantity:"), page3);
 		layout3->addWidget(quantityLabel, row, 0);
 		quantityGroup = new QButtonGroup(this);
@@ -272,7 +303,7 @@ ImportCSVDialog::ImportCSVDialog(bool extra_parameters, Budget *budg, QWidget *p
 		valueQuantityEdit = new EqonomizeValueEdit(1.0, 2, false, false, page3, budget);
 		layout3->addWidget(valueQuantityEdit, row, 4);
 		row++;
-		
+
 		payeeLabel = new QLabel(tr("Payee:"), page3);
 		layout3->addWidget(payeeLabel, row, 0);
 		payeeGroup = new QButtonGroup(this);
@@ -291,9 +322,28 @@ ImportCSVDialog::ImportCSVDialog(bool extra_parameters, Budget *budg, QWidget *p
 		valuePayeeEdit = new QLineEdit(page3);
 		layout3->addWidget(valuePayeeEdit, row, 4);
 		row++;
-		
+
 	}
-	
+
+	tagsLabel = new QLabel(tr("Tags:"), page3);
+	layout3->addWidget(tagsLabel, row, 0);
+	tagsGroup = new QButtonGroup(this);
+	columnTagsButton = new QRadioButton(tr("Column"), page3);
+	tagsGroup->addButton(columnTagsButton);
+	layout3->addWidget(columnTagsButton, row, 1);
+	columnTagsEdit = new QSpinBox(page3);
+	columnTagsEdit->setRange(1, 100);
+	columnTagsEdit->setValue(b_extra ? 8 : 6);
+	columnTagsEdit->setEnabled(false);
+	layout3->addWidget(columnTagsEdit, row, 2);
+	valueTagsButton = new QRadioButton(tr("Value"), page3);
+	tagsGroup->addButton(valueTagsButton);
+	valueTagsButton->setChecked(true);
+	layout3->addWidget(valueTagsButton, row, 3);
+	valueTagsEdit = new QLineEdit(page3);
+	layout3->addWidget(valueTagsEdit, row, 4);
+	row++;
+
 	layout3->addWidget(new QLabel(tr("Comments:"), page3), row, 0);
 	commentsGroup = new QButtonGroup(this);
 	columnCommentsButton = new QRadioButton(tr("Column"), page3);
@@ -320,6 +370,13 @@ ImportCSVDialog::ImportCSVDialog(bool extra_parameters, Budget *budg, QWidget *p
 	layout3->addLayout(layout3_cm, row, 0, 1, 5);
 	row++;
 
+	QHBoxLayout *layoutPreset2 = new QHBoxLayout();
+	layoutPreset2->addStretch(1);
+	savePresetButton = new QPushButton(tr("Save as preset…"), page3);
+	layoutPreset2->addWidget(savePresetButton);
+	layout3->addLayout(layoutPreset2, row, 0, 1, 5);
+	row++;
+
 	layout3->addItem(new QSpacerItem(1, 1, QSizePolicy::Minimum, QSizePolicy::Expanding), row, 0, 1, 5);
 
 	setOption(QWizard::HaveHelpButton, false);
@@ -346,8 +403,11 @@ ImportCSVDialog::ImportCSVDialog(bool extra_parameters, Budget *budg, QWidget *p
 	connect(valueDateButton, SIGNAL(toggled(bool)), valueDateEdit, SLOT(setEnabled(bool)));
 	connect(columnAC1Button, SIGNAL(toggled(bool)), columnAC1Edit, SLOT(setEnabled(bool)));
 	connect(valueAC1Button, SIGNAL(toggled(bool)), valueAC1Edit, SLOT(setEnabled(bool)));
+	connect(valueAC1Button, SIGNAL(toggled(bool)), valueAC1IncomeEdit, SLOT(setEnabled(bool)));
 	connect(columnAC2Button, SIGNAL(toggled(bool)), columnAC2Edit, SLOT(setEnabled(bool)));
 	connect(valueAC2Button, SIGNAL(toggled(bool)), valueAC2Edit, SLOT(setEnabled(bool)));
+	connect(columnTagsButton, SIGNAL(toggled(bool)), columnTagsEdit, SLOT(setEnabled(bool)));
+	connect(valueTagsButton, SIGNAL(toggled(bool)), valueTagsEdit, SLOT(setEnabled(bool)));
 	connect(columnCommentsButton, SIGNAL(toggled(bool)), columnCommentsEdit, SLOT(setEnabled(bool)));
 	connect(valueCommentsButton, SIGNAL(toggled(bool)), valueCommentsEdit, SLOT(setEnabled(bool)));
 	connect(typeGroup, SIGNAL(buttonClicked(int)), this, SLOT(typeChanged(int)));
@@ -357,12 +417,14 @@ ImportCSVDialog::ImportCSVDialog(bool extra_parameters, Budget *budg, QWidget *p
 		connect(columnPayeeButton, SIGNAL(toggled(bool)), columnPayeeEdit, SLOT(setEnabled(bool)));
 		connect(valuePayeeButton, SIGNAL(toggled(bool)), valuePayeeEdit, SLOT(setEnabled(bool)));
 	}
-	
+	connect(presetCombo, SIGNAL(activated(int)), this, SLOT(loadPreset(int)));
+	connect(savePresetButton, SIGNAL(clicked()), this, SLOT(savePreset()));
+
 	page3->adjustSize();
 	page2->setMinimumWidth(page3->minimumSizeHint().width() + 100);
 	page2->setMinimumHeight(page3->minimumSizeHint().height() + 100);
 	page1->setMinimumSize(page2->minimumSize());
-	
+
 	costLabel->hide();
 	valueCostEdit->hide();
 	valueCostButton->hide();
@@ -374,6 +436,275 @@ ImportCSVDialog::ImportCSVDialog(bool extra_parameters, Budget *budg, QWidget *p
 ImportCSVDialog::~ImportCSVDialog() {
 }
 
+void ImportCSVDialog::loadPreset(int index) {
+	s_preset = presetCombo->itemText(index);
+	if(!presets.contains(s_preset)) return;
+	QList<QVariant> preset = presets[s_preset].toList();
+	if(preset.count() < 26) return;
+	typeGroup->button(preset.at(0).toInt())->setChecked(true);
+	typeChanged(preset.at(0).toInt());
+	fileEdit->setText(preset.at(1).toString());
+	rowEdit->setValue(preset.at(2).toInt());
+	QString delimiter = preset.at(3).toString();
+	if(delimiter == ",") delimiterCombo->setCurrentIndex(0);
+	else if(delimiter == "\t") delimiterCombo->setCurrentIndex(1);
+	else if(delimiter == ";") delimiterCombo->setCurrentIndex(2);
+	else if(delimiter == " ") delimiterCombo->setCurrentIndex(3);
+	else {
+		delimiterCombo->setCurrentIndex(4);
+		delimiterEdit->setText(delimiter);
+	}
+	int i = 4;
+	if(preset.at(i).toBool()) {
+		valueDateButton->setChecked(true);
+		valueDateEdit->setDate(preset.at(i + 1).toDate());
+	} else {
+		columnDateButton->setChecked(true);
+		columnDateEdit->setValue(preset.at(i + 1).toInt());
+	}
+	i += 2;
+	if(preset.at(i).toBool()) {
+		valueDescriptionButton->setChecked(true);
+		valueDescriptionEdit->setText(preset.at(i + 1).toString());
+	} else {
+		columnDescriptionButton->setChecked(true);
+		columnDescriptionEdit->setValue(preset.at(i + 1).toInt());
+	}
+	i += 2;
+	if(preset.at(i).toBool()) {
+		valueCostButton->setChecked(true);
+		valueCostEdit->setValue(preset.at(i + 1).toDouble());
+	} else {
+		columnCostButton->setChecked(true);
+		columnCostEdit->setValue(preset.at(i + 1).toInt());
+	}
+	i += 2;
+	if(preset.at(i).toBool()) {
+		valueValueButton->setChecked(true);
+		valueValueEdit->setValue(preset.at(i + 1).toDouble());
+	} else {
+		columnValueButton->setChecked(true);
+		columnValueEdit->setValue(preset.at(i + 1).toInt());
+	}
+	i += 2;
+	if(preset.at(i).toBool()) {
+		valueAC1Button->setChecked(true);
+		qlonglong id = preset.at(i + 1).toLongLong();
+		bool b = false;
+		for(int acc_i = 0; acc_i < valueAC1Edit->count(); acc_i++) {
+			if(((Account*)  valueAC1Edit->itemData(acc_i).value<void*>())->id() == id) {
+				valueAC1Edit->setCurrentIndex(acc_i);
+				b = true;
+				break;
+			}
+		}
+		if(!b) valueAC1Edit->setCurrentIndex(0);
+		id = preset.at(i + 2).toLongLong();
+		b = false;
+		for(int acc_i = 0; acc_i < valueAC1IncomeEdit->count(); acc_i++) {
+			if(((Account*)  valueAC1IncomeEdit->itemData(acc_i).value<void*>())->id() == id) {
+				valueAC1IncomeEdit->setCurrentIndex(acc_i);
+				b = true;
+				break;
+			}
+		}
+		if(!b) valueAC1IncomeEdit->setCurrentIndex(0);
+	} else {
+		columnAC1Button->setChecked(true);
+		columnAC1Edit->setValue(preset.at(i + 1).toInt());
+	}
+	i += 3;
+	if(preset.at(i).toBool()) {
+		valueAC2Button->setChecked(true);
+		qlonglong id = preset.at(i + 1).toLongLong();
+		bool b = false;
+		for(int acc_i = 0; acc_i < valueAC2Edit->count(); acc_i++) {
+			if(((Account*)  valueAC2Edit->itemData(acc_i).value<void*>())->id() == id) {
+				valueAC2Edit->setCurrentIndex(acc_i);
+				b = true;
+				break;
+			}
+		}
+		if(!b) valueAC2Edit->setCurrentIndex(0);
+	} else {
+		columnAC2Button->setChecked(true);
+		columnAC2Edit->setValue(preset.at(i + 1).toInt());
+	}
+	i += 2;
+	if(b_extra) {
+		if(preset.at(i).toBool()) {
+			valueQuantityButton->setChecked(true);
+			valueQuantityEdit->setValue(preset.at(i + 1).toDouble());
+		} else {
+			columnQuantityButton->setChecked(true);
+			columnQuantityEdit->setValue(preset.at(i + 1).toInt());
+		}
+		i += 2;
+		if(preset.at(i).toBool()) {
+			valuePayeeButton->setChecked(true);
+			valuePayeeEdit->setText(preset.at(i + 1).toString());
+		} else {
+			columnPayeeButton->setChecked(true);
+			columnPayeeEdit->setValue(preset.at(i + 1).toInt());
+		}
+		i += 2;
+	} else {
+		i += 4;
+	}
+	if(preset.at(i).toBool()) {
+		valueTagsButton->setChecked(true);
+		valueTagsEdit->setText(preset.at(i + 1).toString());
+	} else {
+		columnTagsButton->setChecked(true);
+		columnTagsEdit->setValue(preset.at(i + 1).toInt());
+	}
+	i += 2;
+	if(preset.at(i).toBool()) {
+		valueCommentsButton->setChecked(true);
+		valueCommentsEdit->setText(preset.at(i + 1).toString());
+	} else {
+		columnCommentsButton->setChecked(true);
+		columnCommentsEdit->setValue(preset.at(i + 1).toInt());
+	}
+	i += 2;
+	createMissingButton->setChecked(preset.at(i).toBool());
+}
+void ImportCSVDialog::savePreset() {
+	QDialog *dialog = new QDialog(this);
+	dialog->setWindowTitle(tr("Save Preset"));
+	QVBoxLayout *box1 = new QVBoxLayout(dialog);
+	QComboBox *presetEdit = new QComboBox(dialog);
+	presetEdit->setMinimumContentsLength(20);
+	presetEdit->setEditable(true);
+	for(QMap<QString, QVariant>::const_iterator it = presets.constBegin(); it != presets.constEnd(); ++it) {
+		presetEdit->addItem(it.key());
+	}
+	presetEdit->setCurrentText(s_preset);
+	box1->addWidget(presetEdit);
+	QDialogButtonBox *buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
+	buttonBox->button(QDialogButtonBox::Ok)->setShortcut(Qt::CTRL | Qt::Key_Return);
+	connect(buttonBox->button(QDialogButtonBox::Cancel), SIGNAL(clicked()), dialog, SLOT(reject()));
+	connect(buttonBox->button(QDialogButtonBox::Ok), SIGNAL(clicked()), dialog, SLOT(accept()));
+	box1->addWidget(buttonBox);
+	if(dialog->exec() == QDialog::Accepted) {
+		QList<QVariant> preset;
+		s_preset = presetEdit->currentText();
+		preset << typeGroup->checkedId();
+		preset << fileEdit->text();
+		preset << rowEdit->value();
+		switch(delimiterCombo->currentIndex()) {
+			case 0: {preset << ","; break;}
+			case 1: {preset << "\t"; break;}
+			case 2: {preset << ";"; break;}
+			case 3: {preset << " "; break;}
+			default: {preset << delimiterEdit->text();}
+		}
+		if(valueDateButton->isChecked()) {
+			preset << true;
+			preset << valueDateEdit->date();
+		} else {
+			preset << false;
+			preset << columnDateEdit->value();
+		}
+		if(valueDescriptionButton->isChecked()) {
+			preset << true;
+			preset << valueDescriptionEdit->text();
+		} else {
+			preset << false;
+			preset << columnDescriptionEdit->value();
+		}
+		if(valueCostButton->isChecked()) {
+			preset << true;
+			preset << valueCostEdit->value();
+		} else {
+			preset << false;
+			preset << columnCostEdit->value();
+		}
+		if(valueValueButton->isChecked()) {
+			preset << true;
+			preset << valueValueEdit->value();
+		} else {
+			preset << false;
+			preset << columnValueEdit->value();
+		}
+		if(valueAC1Button->isChecked()) {
+			preset << true;
+			if(valueAC1Edit->currentIndex() >= 0) {
+				preset << ((Account*)  valueAC1Edit->currentData().value<void*>())->id();
+			} else {
+				preset << (qlonglong) 0;
+			}
+			if(valueAC1IncomeEdit->currentIndex() >= 0) {
+				preset << ((Account*)  valueAC1IncomeEdit->currentData().value<void*>())->id();
+			} else {
+				preset << (qlonglong) 0;
+			}
+		} else {
+			preset << false;
+			preset << columnAC1Edit->value();
+			preset << 0;
+		}
+		if(valueAC2Button->isChecked()) {
+			preset << true;
+			if(valueAC2Edit->currentIndex() >= 0) {
+				preset << ((Account*)  valueAC2Edit->currentData().value<void*>())->id();
+			} else {
+				preset << (qlonglong) 0;
+			}
+		} else {
+			preset << false;
+			preset << columnAC2Edit->value();
+		}
+		if(b_extra) {
+			if(valueQuantityButton->isChecked()) {
+				preset << true;
+				preset << valueQuantityEdit->value();
+			} else {
+				preset << false;
+				preset << columnQuantityEdit->value();
+			}
+			if(valuePayeeButton->isChecked()) {
+				preset << true;
+				preset << valuePayeeEdit->text();
+			} else {
+				preset << false;
+				preset << columnPayeeEdit->value();
+			}
+		} else {
+			preset << true;
+			preset << 1.0;
+			preset << true;
+			preset << QString();
+		}
+		if(valueTagsButton->isChecked()) {
+			preset << true;
+			preset << valueTagsEdit->text();
+		} else {
+			preset << false;
+			preset << columnTagsEdit->value();
+		}
+		if(valueCommentsButton->isChecked()) {
+			preset << true;
+			preset << valueCommentsEdit->text();
+		} else {
+			preset << false;
+			preset << columnCommentsEdit->value();
+		}
+		preset << createMissingButton->isChecked();
+		presets[s_preset] = preset;
+		if(presetEdit->currentIndex() >= 0) {
+			presetCombo->setCurrentIndex(presetEdit->currentIndex());
+		} else {
+			presetCombo->addItem(s_preset);
+			presetCombo->setCurrentIndex(presetCombo->count() - 1);
+			presetCombo->setEnabled(true);
+		}
+		QSettings settings;
+		settings.beginGroup("GeneralOptions");
+		settings.setValue("CSVPresets", presets);
+	}
+	dialog->deleteLater();
+}
 void ImportCSVDialog::onFileChanged(const QString &str) {
 	((QIFWizardPage*) page(1))->setComplete(!str.isEmpty());
 }
@@ -399,37 +730,14 @@ void ImportCSVDialog::typeChanged(int id) {
 	createMissingButton->setChecked(id != ALL_TYPES_ID);
 
 	valueAC1Edit->clear();
+	valueAC1IncomeEdit->clear();
 	valueAC2Edit->clear();
 	if(id < 5) {
 		for(AccountList<AssetsAccount*>::const_iterator it = budget->assetsAccounts.constBegin(); it != budget->assetsAccounts.constEnd(); ++it) {
 			AssetsAccount *aa = *it;
-			if(aa != budget->balancingAccount && aa->accountType() != ASSETS_TYPE_SECURITIES) valueAC2Edit->addItem(aa->name(), qVariantFromValue((void*) aa));
+			if(aa != budget->balancingAccount && aa->accountType() != ASSETS_TYPE_SECURITIES) valueAC2Edit->addItem(aa->nameWithParent(), QVariant::fromValue((void*) aa));
 		}
 	}
-	if(id == 4) {
-		costLabel->show();
-		valueCostEdit->show();
-		columnCostEdit->show();
-		columnCostButton->show();
-		valueCostButton->show();
-		columnCostButton->setChecked(true);
-		valueCostButton->setEnabled(false);
-		valueCostEdit->setEnabled(false);
-		columnValueEdit->setValue(4);
-		columnAC1Edit->setValue(5);
-		columnAC2Edit->setValue(6);
-		columnCommentsEdit->setValue(7);
-	} else {
-		costLabel->hide();
-		valueCostEdit->hide();
-		columnCostEdit->hide();
-		columnCostButton->hide();
-		valueCostButton->hide();
-		columnValueEdit->setValue(3);
-		columnAC1Edit->setValue(4);
-		columnAC2Edit->setValue(5);
-		columnCommentsEdit->setValue(6);
-	}
 	if(b_extra) {
 		quantityLabel->setVisible(id != 2);
 		valueQuantityEdit->setVisible(id != 2);
@@ -441,13 +749,59 @@ void ImportCSVDialog::typeChanged(int id) {
 		columnPayeeEdit->setVisible(id != 2);
 		columnPayeeButton->setVisible(id != 2);
 		valuePayeeButton->setVisible(id != 2);
-		columnCommentsEdit->setValue((id == 2) ? 6 : 8);
+		valueQuantityButton->setChecked(id == 2);
+		valuePayeeButton->setChecked(id == 2);
+	}
+	valueTagsButton->setChecked(id == 2);
+	tagsLabel->setVisible(id != 2);
+	valueTagsEdit->setVisible(id != 2);
+	columnTagsEdit->setVisible(id != 2);
+	columnTagsButton->setVisible(id != 2);
+	valueTagsButton->setVisible(id != 2);
+	valueAC1IncomeEdit->setVisible(id == 3 || id == 4);
+	if(id == 4) {
+		costLabel->show();
+		valueCostEdit->show();
+		columnCostEdit->show();
+		columnCostButton->show();
+		valueCostButton->show();
+		columnCostButton->setChecked(true);
+		valueCostButton->setEnabled(false);
+		valueCostEdit->setEnabled(false);
+		columnValueEdit->setValue(4);
+		columnAC1Edit->setValue(5);
+		columnAC2Edit->setValue(6);
+		if(b_extra) {
+			columnQuantityEdit->setValue(7);
+			columnPayeeEdit->setValue(8);
+			columnTagsEdit->setValue(9);
+			columnCommentsEdit->setValue(10);
+		} else {
+			columnTagsEdit->setValue(7);
+			columnCommentsEdit->setValue(8);
+		}
+	} else {
+		costLabel->hide();
+		valueCostEdit->hide();
+		columnCostEdit->hide();
+		columnCostButton->hide();
+		valueCostButton->hide();
+		columnValueEdit->setValue(3);
+		columnAC1Edit->setValue(4);
+		columnAC2Edit->setValue(5);
 		if(id == 2) {
-			valueQuantityButton->setChecked(true);
-			valuePayeeButton->setChecked(true);
+			columnCommentsEdit->setValue(6);
+		} else if(b_extra) {
+			columnQuantityEdit->setValue(6);
+			columnPayeeEdit->setValue(7);
+			columnTagsEdit->setValue(8);
+			columnCommentsEdit->setValue(9);
+		} else {
+			columnTagsEdit->setValue(6);
+			columnCommentsEdit->setValue(7);
 		}
 	}
-	
+
 	switch(id) {
 		case 0: {
 			typeDescriptionLabel->setText(tr("Imports data as expenses. Costs have positive value. Value is the only required column."));
@@ -457,7 +811,7 @@ void ImportCSVDialog::typeChanged(int id) {
 			if(b_extra) payeeLabel->setText(tr("Payee:"));
 			for(AccountList<ExpensesAccount*>::const_iterator it = budget->expensesAccounts.constBegin(); it != budget->expensesAccounts.constEnd(); ++it) {
 				ExpensesAccount *ea = *it;
-				valueAC1Edit->addItem(ea->name(), qVariantFromValue((void*) ea));
+				valueAC1Edit->addItem(ea->nameWithParent(), QVariant::fromValue((void*) ea));
 			}
 			break;
 		}
@@ -469,7 +823,7 @@ void ImportCSVDialog::typeChanged(int id) {
 			if(b_extra) payeeLabel->setText(tr("Payer:"));
 			for(AccountList<IncomesAccount*>::const_iterator it = budget->incomesAccounts.constBegin(); it != budget->incomesAccounts.constEnd(); ++it) {
 				IncomesAccount *ia = *it;
-				valueAC1Edit->addItem(ia->name(), qVariantFromValue((void*) ia));
+				valueAC1Edit->addItem(ia->nameWithParent(), QVariant::fromValue((void*) ia));
 			}
 			break;
 		}
@@ -480,31 +834,39 @@ void ImportCSVDialog::typeChanged(int id) {
 			AC2Label->setText(tr("To account:"));
 			for(AccountList<AssetsAccount*>::const_iterator it = budget->assetsAccounts.constBegin(); it != budget->assetsAccounts.constEnd(); ++it) {
 				AssetsAccount *aa = *it;
-				if(aa != budget->balancingAccount && aa->accountType() != ASSETS_TYPE_SECURITIES) valueAC1Edit->addItem(aa->name(), qVariantFromValue((void*) *it));
+				if(aa != budget->balancingAccount && aa->accountType() != ASSETS_TYPE_SECURITIES) valueAC1Edit->addItem(aa->nameWithParent(), QVariant::fromValue((void*) *it));
 			}
 			break;
 		}
 		case 3: {
-			typeDescriptionLabel->setText(tr("Imports data as expenses and incomes. Costs have negative value. Value and category are both required columns."));
-			columnAC1Button->setChecked(true);
-			columnAC1Edit->setEnabled(true);
-			valueAC1Edit->setEnabled(false);
-			valueAC1Button->setEnabled(false);
+			typeDescriptionLabel->setText(tr("Imports data as expenses and incomes. Costs have negative value. Value is the only required column."));
 			valueLabel->setText(tr("Value:"));
 			AC1Label->setText(tr("Category:"));
 			AC2Label->setText(tr("Account:"));
+			for(AccountList<ExpensesAccount*>::const_iterator it = budget->expensesAccounts.constBegin(); it != budget->expensesAccounts.constEnd(); ++it) {
+				ExpensesAccount *ea = *it;
+				valueAC1Edit->addItem(ea->nameWithParent(), QVariant::fromValue((void*) ea));
+			}
+			for(AccountList<IncomesAccount*>::const_iterator it = budget->incomesAccounts.constBegin(); it != budget->incomesAccounts.constEnd(); ++it) {
+				IncomesAccount *ia = *it;
+				valueAC1IncomeEdit->addItem(ia->nameWithParent(), QVariant::fromValue((void*) ia));
+			}
 			if(b_extra) payeeLabel->setText(tr("Payee/payer:"));
 			break;
 		}
 		case 4: {
-			typeDescriptionLabel->setText(tr("Imports data as expenses and incomes. Costs and incomes have separate columns. Income, cost, and category are all required columns."));
-			columnAC1Button->setChecked(true);
-			columnAC1Edit->setEnabled(true);
-			valueAC1Edit->setEnabled(false);
-			valueAC1Button->setEnabled(false);
+			typeDescriptionLabel->setText(tr("Imports data as expenses and incomes. Costs and incomes have separate columns. Income and cost both all required columns."));
 			valueLabel->setText(tr("Income:"));
 			AC1Label->setText(tr("Category:"));
 			AC2Label->setText(tr("Account:"));
+			for(AccountList<ExpensesAccount*>::const_iterator it = budget->expensesAccounts.constBegin(); it != budget->expensesAccounts.constEnd(); ++it) {
+				ExpensesAccount *ea = *it;
+				valueAC1Edit->addItem(ea->nameWithParent(), QVariant::fromValue((void*) ea));
+			}
+			for(AccountList<IncomesAccount*>::const_iterator it = budget->incomesAccounts.constBegin(); it != budget->incomesAccounts.constEnd(); ++it) {
+				IncomesAccount *ia = *it;
+				valueAC1IncomeEdit->addItem(ia->nameWithParent(), QVariant::fromValue((void*) ia));
+			}
 			if(b_extra) payeeLabel->setText(tr("Payee/payer:"));
 			break;
 		}
@@ -533,6 +895,12 @@ void ImportCSVDialog::typeChanged(int id) {
 		valueAC1Edit->setEnabled(false);
 		valueAC1Button->setEnabled(false);
 	}
+	if((id == 3 || id == 4) && valueAC1IncomeEdit->count() == 0) {
+		columnAC1Button->setChecked(true);
+		columnAC1Edit->setEnabled(true);
+		valueAC1Edit->setEnabled(false);
+		valueAC1Button->setEnabled(false);
+	}
 	if(valueAC2Edit->count() == 0) {
 		columnAC2Button->setChecked(true);
 		columnAC2Edit->setEnabled(true);
@@ -640,7 +1008,11 @@ void testCSVDate(const QString &str, bool &p1, bool &p2, bool &p3, bool &p4, boo
 		lz = 1;
 		return;
 	}
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
+	QStringList strlist = str.split(separator, Qt::SkipEmptyParts);
+#else
 	QStringList strlist = str.split(separator, QString::SkipEmptyParts);
+#endif
 	if(strlist.count() == 2 && (p1 || p2)) {
 		int i = strlist[1].indexOf('\'');
 		if(i >= 0) {
@@ -808,6 +1180,7 @@ bool ImportCSVDialog::import(bool test, csv_info *ci) {
 	int AC1_c = columnAC1Button->isChecked() ? columnAC1Edit->value() : -1;
 	int AC2_c = columnAC2Button->isChecked() ? columnAC2Edit->value() : -1;
 	int comments_c = columnCommentsButton->isChecked() ? columnCommentsEdit->value() : -1;
+	int tags_c = columnTagsButton->isChecked() ? columnTagsEdit->value() : -1;
 	int payee_c = -1;
 	if(b_extra && columnPayeeButton->isChecked()) payee_c = columnPayeeEdit->value();
 	int quantity_c = -1;
@@ -821,28 +1194,32 @@ bool ImportCSVDialog::import(bool test, csv_info *ci) {
 	min_columns = ncolumns;
 	if(description_c > ncolumns) ncolumns = description_c;
 	if(comments_c > ncolumns) ncolumns = comments_c;
+	if(tags_c > ncolumns) ncolumns = tags_c;
 	if(payee_c > ncolumns) ncolumns = payee_c;
 	if(quantity_c > ncolumns) ncolumns = quantity_c;
-	if((description_c > 0 && (description_c == value_c || description_c == cost_c || description_c == date_c || description_c == AC1_c || description_c == AC2_c || description_c == comments_c || description_c == payee_c || description_c == quantity_c))
-		   || (value_c > 0 && (value_c == date_c || value_c == cost_c || value_c == AC1_c || value_c == AC2_c || value_c == comments_c || value_c == payee_c || value_c == quantity_c))
-		   || (cost_c > 0 && (cost_c == date_c || cost_c == AC1_c || cost_c == AC2_c || cost_c == comments_c || cost_c == payee_c || cost_c == quantity_c))
-		   || (date_c > 0 && (date_c == AC1_c || date_c == AC2_c || date_c == comments_c || date_c == payee_c || date_c == quantity_c))
-		   || (AC1_c > 0 && (AC1_c == AC2_c || AC1_c == comments_c || AC1_c == payee_c || AC1_c == quantity_c))
-		   || (AC2_c > 0 && (AC2_c == comments_c || AC2_c == payee_c || AC2_c == quantity_c))
-		   || (comments_c > 0 && (comments_c == payee_c || comments_c == quantity_c))
-		   || (payee_c > 0 && (payee_c == quantity_c))
+	if((description_c > 0 && (description_c == value_c || description_c == cost_c || description_c == date_c || description_c == AC1_c || description_c == AC2_c || description_c == comments_c || description_c == payee_c || description_c == quantity_c || description_c == tags_c))
+		   || (value_c > 0 && (value_c == date_c || value_c == cost_c || value_c == AC1_c || value_c == AC2_c || value_c == comments_c || value_c == payee_c || value_c == quantity_c || value_c == tags_c))
+		   || (cost_c > 0 && (cost_c == date_c || cost_c == AC1_c || cost_c == AC2_c || cost_c == comments_c || cost_c == payee_c || cost_c == quantity_c || cost_c == tags_c))
+		   || (date_c > 0 && (date_c == AC1_c || date_c == AC2_c || date_c == comments_c || date_c == payee_c || date_c == quantity_c || date_c == tags_c))
+		   || (AC1_c > 0 && (AC1_c == AC2_c || AC1_c == comments_c || AC1_c == payee_c || AC1_c == quantity_c || AC1_c == tags_c))
+		   || (AC2_c > 0 && (AC2_c == comments_c || AC2_c == payee_c || AC2_c == quantity_c || AC2_c == tags_c))
+		   || (comments_c > 0 && (comments_c == payee_c || comments_c == quantity_c || comments_c == tags_c))
+		   || (payee_c > 0 && (payee_c == quantity_c || payee_c == tags_c))
+		   || (tags_c > 0 && (tags_c == quantity_c))
 	  ) {
-		QMessageBox::critical(this, tr("Error"), tr("The same column number is selected multiple times."));
-		return false;
+		if(QMessageBox::warning(this, tr("Warning"), tr("The same column number is selected multiple times. Do you wish to proceed anyway?"), QMessageBox::Yes | QMessageBox::No) != QMessageBox::Yes) {
+			return false;
+		}
 	}
 	bool create_missing = createMissingButton->isChecked() && type != ALL_TYPES_ID;
-	QString description, comments, payee;
+	QString description, comments, payee, tags;
 	double quantity = 1.0;
 	if(!test && description_c < 0) description = valueDescriptionEdit->text();
 	if(!test && comments_c < 0) comments = valueCommentsEdit->text();
 	if(!test && b_extra && quantity_c < 0) quantity = valueQuantityEdit->value();
+	if(!test && tags_c < 0) tags = valueTagsEdit->text();
 	QMap<QString, Account*> eaccounts, iaccounts, aaccounts;
-	Account *ac1 = NULL, *ac2 = NULL;
+	Account *ac1 = NULL, *ac1i = NULL, *ac2 = NULL;
 	if(!test && (AC1_c >= 0 || AC2_c >= 0)) {
 		for(AccountList<ExpensesAccount*>::const_iterator it = budget->expensesAccounts.constBegin(); it != budget->expensesAccounts.constEnd(); ++it) {
 			ExpensesAccount *ea = *it;
@@ -856,11 +1233,12 @@ bool ImportCSVDialog::import(bool test, csv_info *ci) {
 		}
 		for(AccountList<AssetsAccount*>::const_iterator it = budget->assetsAccounts.constBegin(); it != budget->assetsAccounts.constEnd(); ++it) {
 			AssetsAccount *aa = *it;
-			aaccounts[aa->name()] = aa; 
+			aaccounts[aa->name()] = aa;
 		}
 	}
 	if(AC1_c < 0) {
 		if(valueAC1Edit->currentData().isValid()) ac1 = (Account*) valueAC1Edit->currentData().value<void*>();
+		if(valueAC1IncomeEdit->currentData().isValid()) ac1i = (Account*) valueAC1IncomeEdit->currentData().value<void*>();
 	}
 	if(AC2_c < 0) {
 		if(valueAC2Edit->currentData().isValid()) ac2 = (Account*) valueAC2Edit->currentData().value<void*>();
@@ -894,10 +1272,10 @@ bool ImportCSVDialog::import(bool test, csv_info *ci) {
 		QMessageBox::critical(this, tr("Error"), tr("Error reading %1.").arg(url));
 		return false;
 	}
-	
+
 	QFileInfo fileInfo(url);
 	last_document_directory = fileInfo.absoluteDir().absolutePath();
-	
+
 	QTextStream fstream(&file);
 	fstream.setCodec("UTF-8");
 
@@ -912,10 +1290,15 @@ bool ImportCSVDialog::import(bool test, csv_info *ci) {
 	QString line = fstream.readLine();
 	QString new_ac1 = "", new_ac2 = "";
 	QDate curdate = QDate::currentDate();
+	QMap<QDate, qint64> datestamps;
 	while(!line.isNull()) {
 		row++;
 		if((first_row == 0 && !line.isEmpty() && line[0] != '#') || (first_row > 0 && row >= first_row && !line.isEmpty())) {
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
+			QStringList columns = line.split(delimiter, Qt::KeepEmptyParts);
+#else
 			QStringList columns = line.split(delimiter, QString::KeepEmptyParts);
+#endif
 			for(QStringList::Iterator it = columns.begin(); it != columns.end(); ++it) {
 				int i = 0;
 				while(i < (int) (*it).length() && ((*it)[i] == ' ' || (*it)[i] == '\t')) {
@@ -941,14 +1324,16 @@ bool ImportCSVDialog::import(bool test, csv_info *ci) {
 								(*it2).truncate(i);
 								*it += delimiter;
 								*it += *it2;
-								columns.erase(it2);
+								it2 = columns.erase(it2);
+								it = it2;
+								it--;
 								break;
 							}
 							*it += delimiter;
 							*it += *it2;
-							columns.erase(it2);
-							it2 = it;
-							++it2;
+							it2 = columns.erase(it2);
+							it = it2;
+							it--;
 						}
 					}
 				}
@@ -1189,6 +1574,9 @@ bool ImportCSVDialog::import(bool test, csv_info *ci) {
 				if(success && comments_c > 0) {
 					comments = columns[comments_c - 1];
 				}
+				if(success && tags_c > 0) {
+					tags = columns[tags_c - 1];
+				}
 				if(success && payee_c > 0) {
 					payee = columns[payee_c - 1];
 				}
@@ -1264,8 +1652,8 @@ bool ImportCSVDialog::import(bool test, csv_info *ci) {
 								iaccounts[ac1->name()] = ac1;
 							}
 						} else if(type == 2) {
-							ac1 = new AssetsAccount(budget, ASSETS_TYPE_CASH, new_ac1); 
-							budget->addAccount(ac1); 
+							ac1 = new AssetsAccount(budget, ASSETS_TYPE_CASH, new_ac1);
+							budget->addAccount(ac1);
 							aaccounts[ac1->name()] = ac1;
 						}
 						new_ac1 = "";
@@ -1307,7 +1695,7 @@ bool ImportCSVDialog::import(bool test, csv_info *ci) {
 								trans = new Expense(budget, -value, date, (ExpensesAccount*) ac1, (AssetsAccount*) ac2, description, comments);
 								((Expense*) trans)->setPayee(payee);
 							} else {
-								trans = new Income(budget, value, date, (IncomesAccount*) ac1, (AssetsAccount*) ac2, description, comments);
+								trans = new Income(budget, value, date, AC1_c < 0 ? (IncomesAccount*) ac1i : (IncomesAccount*) ac1, (AssetsAccount*) ac2, description, comments);
 								((Income*) trans)->setPayer(payee);
 							}
 							successes++;
@@ -1330,12 +1718,17 @@ bool ImportCSVDialog::import(bool test, csv_info *ci) {
 						}
 					}
 					if(trans) {
+						trans->readTags(tags);
 						trans->setQuantity(quantity);
 						if(trans->date() > curdate) {
+							trans->setTimestamp(datestamps.contains(QDate::currentDate()) ? datestamps[QDate::currentDate()] + 1 : DATE_TO_MSECS(QDate::currentDate()) / 1000);
+							datestamps[QDate::currentDate()] = trans->timestamp();
 							budget->addScheduledTransaction(new ScheduledTransaction(budget, trans, NULL));
 						} else {
+							trans->setTimestamp(datestamps.contains(trans->date()) ? datestamps[trans->date()] + 1 : DATE_TO_MSECS(trans->date()) / 1000);
+							datestamps[trans->date()] = trans->timestamp();
 							budget->addTransaction(trans);
-						}						
+						}
 					}
 				} else {
 					failed++;
@@ -1389,7 +1782,7 @@ void ImportCSVDialog::accept() {
 		return;
 	}
 	if(ci.value_format < 0 || ps > 1) {
-		QDialog *dialog = new QDialog(this, 0);
+		QDialog *dialog = new QDialog(this);
 		dialog->setWindowTitle(tr("Specify Format"));
 		dialog->setModal(true);
 		QVBoxLayout *box1 = new QVBoxLayout(dialog);
@@ -1451,8 +1844,7 @@ void ImportCSVDialog::accept() {
 			grid->addWidget(valueFormatCombo, ps > 1 ? 2 : 1, 1);
 		}
 		QDialogButtonBox *buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
-		buttonBox->button(QDialogButtonBox::Cancel)->setShortcut(Qt::CTRL | Qt::Key_Return);
-		buttonBox->button(QDialogButtonBox::Cancel)->setDefault(true);
+		buttonBox->button(QDialogButtonBox::Ok)->setShortcut(Qt::CTRL | Qt::Key_Return);
 		connect(buttonBox->button(QDialogButtonBox::Cancel), SIGNAL(clicked()), dialog, SLOT(reject()));
 		connect(buttonBox->button(QDialogButtonBox::Ok), SIGNAL(clicked()), dialog, SLOT(accept()));
 		box1->addWidget(buttonBox);
