@@ -42,21 +42,35 @@
 #include "budget.h"
 #include "eqonomize.h"
 
-int main(int argc, char **argv) {	
-	
+QTranslator translator, translator_qt, translator_qtbase;
+
+int main(int argc, char **argv) {
+
 	QApplication app(argc, argv);
 	app.setApplicationName("eqonomize");
 	app.setApplicationDisplayName("Eqonomize!");
 	app.setOrganizationName("Eqonomize");
 	app.setApplicationVersion(VERSION);
-	
+
 	setlocale(LC_MONETARY, "");
-	
-	QTranslator translator;
-	if(translator.load(QLocale(), QLatin1String("eqonomize"), QLatin1String("_"), QLatin1String(TRANSLATIONS_DIR))) {
-		app.installTranslator(&translator);
+
+	QSettings settings;
+	settings.beginGroup("GeneralOptions");
+	QString slang = settings.value("language", QString()).toString();
+
+	EqonomizeTranslator eqtr;
+	app.installTranslator(&eqtr);
+	if(!slang.isEmpty()) {
+		if(translator.load(QLatin1String("eqonomize") + QLatin1String("_") + slang, QLatin1String(TRANSLATIONS_DIR))) app.installTranslator(&translator);
+		if(translator_qt.load("qt_" + slang, QLibraryInfo::location(QLibraryInfo::TranslationsPath))) app.installTranslator(&translator_qt);
+		if(translator_qtbase.load("qtbase_" + slang, QLibraryInfo::location(QLibraryInfo::TranslationsPath))) app.installTranslator(&translator_qtbase);
+	} else {
+		if(translator.load(QLocale(), QLatin1String("eqonomize"), QLatin1String("_"), QLatin1String(TRANSLATIONS_DIR))) app.installTranslator(&translator);
+		if(translator_qt.load(QLocale(), QLatin1String("qt"), QLatin1String("_"), QLibraryInfo::location(QLibraryInfo::TranslationsPath))) app.installTranslator(&translator_qt);
+		if(translator_qtbase.load(QLocale(), QLatin1String("qtbase"), QLatin1String("_"), QLibraryInfo::location(QLibraryInfo::TranslationsPath))) app.installTranslator(&translator_qtbase);
 	}
-	
+
+
 	QCommandLineParser *parser = new QCommandLineParser();
 	QCommandLineOption eOption(QStringList() << "e" << "expenses", QApplication::tr("Start with expenses list displayed"));
 	parser->addOption(eOption);
@@ -69,7 +83,7 @@ int main(int argc, char **argv) {
 	parser->addPositionalArgument("url", QApplication::tr("Document to open"), "[url]");
 	parser->addHelpOption();
 	parser->process(app);
-			
+
 #if (QT_VERSION >= QT_VERSION_CHECK(5, 5, 0))
 	QString lockpath = QStandardPaths::writableLocation(QStandardPaths::AppConfigLocation);
 #else
@@ -108,8 +122,6 @@ int main(int argc, char **argv) {
 		}
 	}
 
-	QSettings settings;
-	settings.beginGroup("GeneralOptions");
 	QString url = settings.value("lastURL", QString()).toString();
 	settings.endGroup();
 
@@ -126,7 +138,7 @@ int main(int argc, char **argv) {
 		QString error = budget->loadFile(u.toLocalFile(), errors);
 		if(!error.isNull()) {qWarning() << error; return EXIT_FAILURE;}
 		if(!errors.isEmpty()) qWarning() << errors;
-		errors = QString::null;
+		errors = QString();
 		if(budget->sync(error, errors, true, true)) {
 			if(!errors.isEmpty()) qWarning() << errors;
 			error = budget->saveFile(u.toLocalFile());
@@ -137,15 +149,15 @@ int main(int argc, char **argv) {
 		}
 		return 0;
 	}
-	
-#ifndef LOAD_EQZICONS_FROM_FILE	
+
+#ifndef LOAD_EQZICONS_FROM_FILE
 	if(QIcon::themeName().isEmpty() || !QIcon::hasThemeIcon("eqz-account")) {
 		QIcon::setThemeSearchPaths(QStringList(ICON_DIR));
 		QIcon::setThemeName("EQZ");
 	}
 #endif
 	app.setWindowIcon(LOAD_APP_ICON("eqonomize"));
-	
+
 	Eqonomize *win = new Eqonomize();
 	win->setCommandLineParser(parser);
 	if(parser->isSet(eOption)) {
@@ -154,10 +166,10 @@ int main(int argc, char **argv) {
 		win->showIncomes();
 	} else if(parser->isSet(tOption)) {
 		win->showTransfers();
-	}	
+	}
 	win->show();
 	app.processEvents();
-	
+
 	QStringList args = parser->positionalArguments();
 	if(args.count() > 0) {
 		win->openURL(QUrl::fromUserInput(args.at(0)));
@@ -174,7 +186,7 @@ int main(int argc, char **argv) {
 #endif
 
 	args.clear();
-	
+
 	return app.exec();
 
 }
